#!/usr/bin/python2.7
# -*-coding:Utf-8-*
from netCDF4 import Dataset
import numpy as np
from matplotlib.path import Path
import cPickle as pickle

__author__   = 'Thomas Roc'
__email__    = 'thomas.roc@gmail.co.uk'
__created__  = datetime.datetime(2014, 12, 5)
__modified__ = datetime.datetime(2014, 12, 5)
__version__  = "0.0"
__status__   = "Under Development"

path_data = "/media/Data/TPXO_tide_model/tpxo_8.0/"
tpxofile_data={}
tpxofile_data['grid'] = {"atlas6": path_data + 'grid_tpxo8_atlas6.nc', \
                         "atlas30": path_data + 'grid_tpxo8atlas_30.nc'}
tpxofile_data['elev'] = {"k1": path_data + 'hf.k1_tpxo8_atlas_30c.nc', \
                         "k2": path_data + 'hf.k2_tpxo8_atlas_30c.nc', \
                         "m2": path_data + 'hf.m2_tpxo8_atlas_30c.nc', \
                         "m4": path_data + 'hf.m4_tpxo8_atlas_30c.nc', \
                         "mf": path_data + 'hf.mf_tpxo8_atlas_6.nc',   \
                         "mm": path_data + 'hf.mm_tpxo8_atlas_6.nc',   \
                        "mn4": path_data + 'hf.mn4_tpxo8_atlas_6.nc',  \
                        "ms4": path_data + 'hf.ms4_tpxo8_atlas_6.nc',  \
                         "n2": path_data + 'hf.n2_tpxo8_atlas_30c.nc', \
                         "o1": path_data + 'hf.o1_tpxo8_atlas_30c.nc', \
                         "p1": path_data + 'hf.p1_tpxo8_atlas_30c.nc', \
                         "q1": path_data + 'hf.q1_tpxo8_atlas_30c.nc', \
                         "s2": path_data + 'hf.s2_tpxo8_atlas_30c.nc'  }
tpxofile_data['vel'] =  {"k1": path_data + 'uv.k1_tpxo8_atlas_30c.nc', \
                         "k2": path_data + 'uv.k2_tpxo8_atlas_30c.nc', \
                         "m2": path_data + 'uv.m2_tpxo8_atlas_30c.nc', \
                         "m4": path_data + 'uv.m4_tpxo8_atlas_30c.nc', \
                         "mf": path_data + 'uv.mf_tpxo8_atlas_6.nc',   \
                         "mm": path_data + 'uv.mm_tpxo8_atlas_6.nc',   \
                        "mn4": path_data + 'uv.mn4_tpxo8_atlas_6.nc',  \
                        "ms4": path_data + 'uv.ms4_tpxo8_atlas_6.nc',  \
                         "n2": path_data + 'uv.n2_tpxo8_atlas_30c.nc', \
                         "o1": path_data + 'uv.o1_tpxo8_atlas_30c.nc', \
                         "p1": path_data + 'uv.p1_tpxo8_atlas_30c.nc', \
                         "q1": path_data + 'uv.q1_tpxo8_atlas_30c.nc', \
                         "s2": path_data + 'uv.s2_tpxo8_atlas_30c.nc'  }

def read_tpxo_data(lon,lat, harmonic_names, pickle_name='tpxo8_data.p',
                   tpxofile_data_dict=tpxofile_data):
    """
Reads data from TPXO files and interpolates them onto the ROMS grid

Inputs:
------
  -lon, lat: ndarray type objects of (n,m) shape containing coordinates in deg.
  - harmonic_names: list of harmonic constituents, list
Options:
-------
  - pickle_name: name of the pickle file backup
  - tpxofile_data_dict: dictionnary type object containing the paths of 
                        the tpxo netcdf data files.
    """

    lonmin1 = lon.min()
    lonmax1 = lon.max()
    latmin1 = lat.min()
    latmax1 = lat.max()

    if (lonmin1 < 0):
        lon = np.mod(lon, 360.0)

    if ((lonmin1 > 0) or (lonmax1 < 0)):
        pt1=(lon[0,0],lat[0,0])
        pt2=(lon[-1,0],lat[-1,0])
        pt3=(lon[-1,-1],lat[-1,-1])
        pt4=(lon[0,-1],lat[0,-1])
        bnd_vert = [pt1, pt2, pt3, pt4]
    else:
        pt1=(lon[0,0],lat[0,0])
        pt2=(lon[-1,0],lat[-1,0])
        pt3=(lon[-1,-1],lat[-1,-1])
        pt4=(lon[0,-1],lat[0,-1])
        pt5=(0.0,((lat[0,0]+lat[0,-1])/2.0))
        pt6=(0.0,((lat[-1,0]+lat[-1,-1])/2.0))
        pt7=(360.0,((lat[0,0]+lat[0,-1])/2.0))
        pt8=(360.0,((lat[-1,0]+lat[-1,-1])/2.0))
        bnd_vert_1= [pt1, pt2, pt7, pt8]
        bnd_vert_2= [pt6, pt6, pt3, pt4]

    lonmin = lon.min()
    lonmax = lon.max()
    latmin = lat.min()
    latmax = lat.max()

    grid6 = Dataset(tpxofile_data['grid']['atlas6'])
    grid30 = Dataset(tpxofile_data['grid']['atlas30'])

    X6 = grid6.variables['lon_z'][:]
    Y6 = grid6.variables['lat_z'][:]
    X30 = grid30.variables['lon_z'][:]
    Y30 = grid30.variables['lat_z'][:]

    I6 = []
    J6 = []
    I30 = []
    J30 = []

    ##correction when grid spans over longitude 0
    if ((lonmin1 > 0) or (lonmax1 < 0)):
        for i in range(0, len(X6)):
            if ((X6[i]<=(lonmax+0.5))and(X6[i]>=(lonmin-0.5))):
                I6.append(i)
        for j in range(0, len(Y6)):
             if ((Y6[j]<=(latmax+0.5))and(Y6[j]>=(latmin-0.5))):
                J6.append(j)
        for i in range(0, len(X30)):
            if ((X30[i]<=(lonmax+0.5))and(X30[i]>=(lonmin-0.5))):
                I30.append(i)
        for j in range(0, len(Y30)):
             if ((Y30[j]<=(latmax+0.5))and(Y30[j]>=(latmin-0.5))):
                J30.append(j)
    else :
        for i in range(0, len(X6)):
            if (((X6[i]<=(lonmax1 + 0.5))and(X6[i]>=0.0))\
            or ((X6[i]<=360.0)and(X6[i]>=np.mod(lonmin1-0.5,360.0)))):
                I6.append(i)
        for j in range(0, len(Y6)): 
            if ((Y6[i,j]<=(latmax+0.5))and(Y6[i,j]>=(latmin-0.5))):
                J6.append(j)
        for i in range(0, len(X30)):
            if (((X30[i]<=(lonmax1 + 0.5))and(X30[i]>=0.0))\
            or ((X30[i]<=360.0)and(X30[i]>=np.mod(lonmin1-0.5,360.0)))):
                I30.append(i)
        for j in range(0, len(Y30)): 
            if ((Y30[i,j]<=(latmax+0.5))and(Y30[i,j]>=(latmin-0.5))):
                J30.append(j)            
 
    coord = ['z', 'u', 'v']
    var = ['h', 'u', 'v']
    TPXO = {}
    test=''

    for harmo in harmonic_names: 
        TPXO[harmo.lower()] = {}
        for v in var:
            TPXO[harmo.lower()][v] = {}

    for harmonic in harmonic_names:
        harmonic = harmonic.lower()
        print("Extracting {} components".format(harmonic))
        #print ('K=' + str(K))

        elev = Dataset(tpxofile_data['elev'][harmonic])
        vel = Dataset(tpxofile_data['vel'][harmonic])
        files = [elev,vel,vel]
        if harmonic in ['ms4', 'mn4', 'mf', 'mm']:
            grid = grid6
            I=I6
            J=J6
        else:
            grid = grid30
            I=I30
            J=J30

        for step in range(0, len(var)):
            #print ('step=' + str(step))

            X1 = files[step].variables['lon_' + coord[step]][:]
            Y1 = files[step].variables['lat_' + coord[step]][:]
            [Y,X]=np.meshgrid(Y1,X1)
            H = grid.variables['h' + coord[step]][:]
            Z = np.vectorize(complex)(files[step].variables[var[step] + 'Re'], \
                                      files[step].variables[var[step] + 'Im'])

            x = np.zeros((len(I), len(J)))
            y = np.zeros((len(I), len(J)))
            h = np.zeros((len(I), len(J)))
            z = np.zeros((len(I), len(J)))            
            z = np.vectorize(complex)(z, z)

            (row, col) = x.shape
            for i in range(0, row):
                for j in range(0, col):
                    if var[step] == 'h':
                        x[i,j] = X[(I[i]),J[j]]
                        y[i,j] = Y[(I[i]),J[j]]                
                        h[i,j] = H[(I[i]),J[j]]
                        z[i,j] = Z[(I[i]),J[j]]/1000.0
                    else:                       
                        x[i,j] = X[I[i],J[j]]
                        y[i,j] = Y[I[i],J[j]]                
                        h[i,j] = H[I[i],J[j]]
                        z[i,j] = Z[I[i],J[j]]/10000.0

            x_pts, y_pts = x.flatten(), y.flatten()
            points = np.vstack((x_pts,y_pts)).T

            #correction when grid spans over longitude 0
            if ((lonmin1 > 0) or (lonmax1 < 0)):
                mask = Path(bnd_vert).contains_points(points)
                mask = mask.reshape((len(I), len(J)))
            else:
                mask_1 = Path(bnd_vert_1).contains_points(points)
                mask_1 = mask_1.reshape((len(I), len(J)))   
                mask_2 = Path(bnd_vert_2).contains_points(points)
                mask_2 = mask_2.reshape((len(I), len(J)))
                mask = (mask_1 + mask_2)
                mask = np.putmask(mask, mask>1, 1)             

            TPXO[harmonic][var[step]] = {'x': x, 'y': y, 'h': h, 'mask': mask, 'z': z}
            if var[step] != 'h':
                 TPXO[harmonic][var[step]]['z'][:,:] = z[:,:] / h[:,:]

    #Save TPXO file in pickle
    print("Pickling tidal data im {} ".format(pickle_name))
    pickle.dump(TPXO, open(pickle_name, "wb"))  
      
    return TPXO
